﻿using UnityEngine;
using System.Collections;
using System.Text;
using System;
using System.Collections.Generic;
using System.Linq;

/* /////////////////////////////////////////////////////////////////////////////////////////////////
https://www.patreon.com/user?u=33893211

Licensed under CC BY-SA after EarlyAccess ended. (see https://creativecommons.org/licenses/by-sa/4.0/)

///////////////////////////////////////////////////////////////////////////////////////////////// */

namespace PluginIdea
{
    class SendData
    {
        public float[] samples;
    }

    public class SystemAudioAction : MVRScript
    {
        private JSONStorableFloat sampleMultNum;
        private JSONStorableFloat rollingMaxSample;
        private FloatTrigger SystemRealTimeAudioAmpTrigger;
        private JSONStorableStringChooser lipSyncTriggerTypes;

        private JSONStorableFloat minLipAmount;
        private JSONStorableFloat maxLipAmount;
        private JSONStorableFloat updateSpeed;

        private List<string> lipSyncTriggerTypesList = new List<string> { "SimpleAudioAmp" };

        private float triggerTimer;
        private float triggerDur = 0.025f;

        private DAZMorph mouthOpen_morph;
        private float currentAmp;
        private float currentTargetMorphValue;
        private float currentMorphValue;

        public override void Init()
        {
            base.Init();

            //get simple mouthopen morph
            SuperController.singleton.PauseSimulation(3, "Loading " + this.containingAtom.name);
            JSONStorable storableByID = this.containingAtom.GetStorableByID("geometry");
            if (storableByID != null)
            {
                DAZCharacterSelector dazcharacterSelector = storableByID as DAZCharacterSelector;
                GenerateDAZMorphsControlUI morphsControlUI = dazcharacterSelector.morphsControlUI;
                if (morphsControlUI != null)
                {
                    List<DAZMorph> morphs = morphsControlUI.GetMorphs();
                    this.mouthOpen_morph = morphs.FirstOrDefault(x => x.region == "Pose Controls/Head/Mouth" && x.morphName == "PHMMouthOpen");
                    if (this.mouthOpen_morph != null)
                    {
                        this.mouthOpen_morph.SetValue(0);
                        this.mouthOpen_morph.isTransient = false;
                        this.mouthOpen_morph.isRuntime = false;
                    }
                    else
                    {
                        SuperController.LogMessage("mouth open morph not found");
                    }
                }
            }

            //trigger
            SimpleTriggerHandler.LoadAssets();

            SystemRealTimeAudioAmpTrigger = new FloatTrigger(this, "SystemRealTimeAudioAmpTrigger");
            Utils.SetupButton(this, "Set Audio Action", SystemRealTimeAudioAmpTrigger.OpenPanel, false);

            sampleMultNum = Utils.SetupSliderFloat(this, "SampleMultNum", 1f, 0.3f, 5f, false);
            rollingMaxSample = Utils.SetupSliderFloat(this, "rollingMaxSample", 1f, 1f, 5f, false);

            Utils.SetupSpacer(this, 30, false);
            lipSyncTriggerTypes = Utils.SetupStringChooser(this, "LipSyncType", lipSyncTriggerTypesList, 0, false);
            minLipAmount = Utils.SetupSliderFloat(this, "MinLipAmount", -0.5f, -0.5f, 2f, false);
            maxLipAmount = Utils.SetupSliderFloat(this, "MaxLipAmount", 1f, -0.5f, 2f, false);
            updateSpeed = Utils.SetupSliderFloat(this, "UpdateSpeed", 100, 0, 300, false);

            Utils.SetupInfoText(this, "SystemAudioAction 1.1 \n\n" +
                "Set Audio Action: set the application target of audio response.\n\n" +
                "SampleMultNum: The multiplier of audio effect, the larger the multiplier, the stronger the effect.\n\n" +
                "RollingMaxSample: the maximum number of audio samples skipped each time. The larger the setting, the later the audio response.\n\n" +
                "MinLipAmount: the minimum opening of the mouth.\n\n" +
                "MaxLipAmount: the maximum opening of the mouth.\n\n" +
                "UpdateSpeed: mouth response speed.", 1000, true);
        }

        private void Update()
        {
            triggerTimer += Time.deltaTime;
            if (triggerTimer > triggerDur)
            {
                triggerTimer = 0;

                try
                {
                    if (UDPReceive.Instance.dataReveived != null)
                    {
                        string sendDataStr = Encoding.Default.GetString(UDPReceive.Instance.dataReveived);
                        SendData sendData = JsonUtility.FromJson<SendData>(sendDataStr);

                        currentAmp = GetAmp(sendData.samples) * 5;
                        currentTargetMorphValue = currentAmp - 0.5f;
                        currentTargetMorphValue = Mathf.Clamp(currentTargetMorphValue, minLipAmount.val, maxLipAmount.val);

                        SystemRealTimeAudioAmpTrigger.Trigger(currentAmp);
                    }
                }
                catch (System.Exception e)
                {
                    SuperController.LogError(e.Message);
                    return;
                }
            }

            currentMorphValue = Mathf.Lerp(currentMorphValue, currentTargetMorphValue, updateSpeed.val * Time.deltaTime);
        }
        private float GetAmp(float[] samples)
        {

            float ampMultiplier = 0f;
            for (int j = 0; j < samples.Length; j++)
            {
                samples[j] /= rollingMaxSample.val + 1E-06f;
                samples[j] *= sampleMultNum.val;
                ampMultiplier += samples[j];
            }
            ampMultiplier /= (float)samples.Length;
            return ampMultiplier;
        }


        private void FixedUpdate()
        {
            this.mouthOpen_morph.SetValue(currentMorphValue);
        }

        private void OnDestroy()
        {
            UDPReceive.Instance.IsAlive = false;
        }
    }
}

